@@ -19,6 +19,8 @@
 Provide configuration values for Kinetics Toolkit's inner working.
 
 """
+from __future__ import annotations
+
 
 __author__ = "Félix Chénier"
 __copyright__ = "Copyright (C) 2020 Félix Chénier"
@@ -29,10 +31,9 @@ __license__ = "Apache 2.0"
 import os
 import warnings
 import platform
-from typing import List
 
 
-def __dir__() -> List[str]:
+def __dir__() -> list[str]:
     return [
         "root_folder",
         "home_folder",
