@@ -19,6 +19,7 @@
 Identify cycles and time-normalize data.
 
 """
+from __future__ import annotations
 
 __author__ = "Félix Chénier"
 __copyright__ = "Copyright (C) 2020 Félix Chénier"
@@ -28,8 +29,13 @@ __license__ = "Apache 2.0"
 
 import numpy as np
 from kineticstoolkit.timeseries import TimeSeries, TimeSeriesEvent
+from kineticstoolkit.exceptions import (
+    check_types,
+    TimeSeriesEventNotFoundError,
+)
 import warnings
-from typing import List, Dict, Tuple, Sequence, Optional
+from typing import Optional
+from numpy.typing import ArrayLike
 
 
 def __dir__():
@@ -46,13 +52,13 @@ def detect_cycles(
     ts: TimeSeries,
     data_key: str,
     *,
-    event_names: Sequence[str] = ["phase1", "phase2"],
-    thresholds: Sequence[float] = [0.0, 1.0],
-    directions: Sequence[str] = ["rising", "falling"],
-    min_durations: Sequence[float] = [0.0, 0.0],
-    max_durations: Sequence[float] = [np.Inf, np.Inf],
-    min_peak_heights: Sequence[float] = [-np.Inf, -np.Inf],
-    max_peak_heights: Sequence[float] = [np.Inf, np.Inf],
+    event_names: list[str] = ["phase1", "phase2"],
+    thresholds: list[float] = [0.0, 1.0],
+    directions: list[str] = ["rising", "falling"],
+    min_durations: list[float] = [0.0, 0.0],
+    max_durations: list[float] = [np.Inf, np.Inf],
+    min_peak_heights: list[float] = [-np.Inf, -np.Inf],
+    max_peak_heights: list[float] = [np.Inf, np.Inf],
 ) -> TimeSeries:
     """
     Detect cycles in a TimeSeries based on a dual threshold approach.
@@ -102,6 +108,8 @@ def detect_cycles(
         A copy of `ts` with the events added.
 
     """
+    check_types(detect_cycles, locals())
+
     # lowercase directions[0] once
     directions[0] = directions[0].lower()  # type: ignore
     if directions[0] != "rising" and directions[0] != "falling":
@@ -190,7 +198,7 @@ def time_normalize(
     event_name2: str,
     *,
     n_points: int = 100,
-    span: Optional[Sequence[int]] = None,
+    span: Optional[list[int]] = None,
 ) -> TimeSeries:
     """
     Time-normalize cycles in a TimeSeries.
@@ -237,6 +245,8 @@ def time_normalize(
     119, 254, etc. For each cycle, events outside the 0-100% spans are ignored.
 
     """
+    check_types(time_normalize, locals())
+
     # Optional span
     if span is None:
         span = [0, n_points]
@@ -245,8 +255,6 @@ def time_normalize(
     if len(ts.events) < 2:
         raise (ValueError("No cycle can be defined from these event names."))
 
-    i_cycle = 0
-
     # Initialize the destination TimeSeries
     dest_ts = ts.copy()
     dest_ts.events = []
@@ -255,24 +263,36 @@ def time_normalize(
     else:
         dest_ts.time_info["Unit"] = f"1/{n_points}"
 
-    dest_data = {}  # type: Dict[str, List[np.ndarray]]
-    dest_data_shape = {}  # type: Dict[str, Tuple[int, ...]]
+    dest_data = {}  # type: dict[str, list[np.ndarray]]
+    dest_data_shape = {}  # type: dict[str, tuple[int, ...]]
 
     # Go through all cycles
+    i_cycle = 0
+    break_now = False
     while True:
 
         # Get the begin time for this cycle
-        begin_time = ts.get_event_time(event_name1, i_cycle)
+        try:
+            event_index = ts._get_event_index(event_name1, i_cycle)
+        except TimeSeriesEventNotFoundError:
+            break_now = True
+        else:
+            begin_time = ts.events[event_index].time
 
         # Get the end time for this cycle
         end_cycle = 0
-        end_time = ts.get_event_time(event_name2, end_cycle)
+        end_time = ts.events[ts._get_event_index(event_name2, end_cycle)].time
         while end_time <= begin_time:
             end_cycle += 1
-            end_time = ts.get_event_time(event_name2, end_cycle)
+            try:
+                end_time = ts.events[
+                    ts._get_event_index(event_name2, end_cycle)
+                ].time
+            except TimeSeriesEventNotFoundError:
+                break_now = True
+                break
 
-        # We are done. Quit the loop.
-        if np.isnan(begin_time) or np.isnan(end_time):
+        if break_now:
             break
 
         # Get the extended begin and end times considering relative_span
@@ -339,10 +359,10 @@ def time_normalize(
         # Add event_name1 at the beginning and end (duplicates will be
         # cancelled at the end)
         dest_ts = dest_ts.add_event(
-            -span[0] + i_cycle * (span[1] - span[0]), event_name1
+            float(-span[0] + i_cycle * (span[1] - span[0])), event_name1
         )
         dest_ts = dest_ts.add_event(
-            -span[0] + n_points + i_cycle * (span[1] - span[0]), "_"
+            float(-span[0] + n_points + i_cycle * (span[1] - span[0])), "_"
         )
 
         # Add the other events
@@ -368,7 +388,6 @@ def time_normalize(
         i_cycle += 1
 
     n_cycles = i_cycle
-
     # Put back dest_time and dest_data in dest_ts
     dest_ts.time = 1.0 * np.arange(n_cycles * (span[1] - span[0]))
     for key in ts.data:
@@ -383,7 +402,7 @@ def time_normalize(
     return dest_ts
 
 
-def stack(ts: TimeSeries, *, n_points: int = 100) -> Dict[str, np.ndarray]:
+def stack(ts: TimeSeries, *, n_points: int = 100) -> dict[str, np.ndarray]:
     """
     Stack time-normalized TimeSeries' data into a dict of arrays.
 
@@ -402,13 +421,15 @@ def stack(ts: TimeSeries, *, n_points: int = 100) -> Dict[str, np.ndarray]:
 
     Returns
     -------
-    Dict[str, np.ndarray]
+    dict[str, np.ndarray]
 
     See Also
     --------
     ktk.cycles.unstack
 
     """
+    check_types(stack, locals())
+
     if np.mod(len(ts.time), n_points) != 0:
         raise (
             ValueError("It seems that this TimeSeries is not time-normalized.")
@@ -424,7 +445,7 @@ def stack(ts: TimeSeries, *, n_points: int = 100) -> Dict[str, np.ndarray]:
     return data
 
 
-def unstack(data: Dict[str, np.ndarray], /) -> TimeSeries:
+def unstack(data: dict[str, ArrayLike], /) -> TimeSeries:
     """
     Unstack time-normalized data from a dict of arrays to a TimeSeries.
 
@@ -447,12 +468,15 @@ def unstack(data: Dict[str, np.ndarray], /) -> TimeSeries:
     ktk.cycles.stack
 
     """
+    check_types(unstack, locals())
+
     ts = TimeSeries()
     for key in data.keys():
-        current_shape = data[key].shape
+        current_data = np.array(data[key])
+        current_shape = current_data.shape
         n_cycles = current_shape[0]
         n_points = current_shape[1]
-        ts.data[key] = data[key].reshape([n_cycles * n_points], order="C")
+        ts.data[key] = current_data.reshape([n_cycles * n_points], order="C")
     ts.time = np.arange(n_cycles * n_points)
     ts.time_info["Unit"] = ""
     return ts
@@ -465,7 +489,7 @@ def unstack(data: Dict[str, np.ndarray], /) -> TimeSeries:
 #
 # def stack_events(
 #         ts: TimeSeries, /,
-#         n_points: int = 100) -> Dict[str, np.ndarray]:
+#         n_points: int = 100) -> dict[str, np.ndarray]:
 #     """
 #     Stack time-normalized TimeSeries' events into a dict of arrays.
 
@@ -489,7 +513,7 @@ def unstack(data: Dict[str, np.ndarray], /) -> TimeSeries:
 
 #     Returns
 #     -------
-#     Dict[str, np.ndarray]
+#     dict[str, np.ndarray]
 
 #     Example
 #     -------
@@ -511,7 +535,7 @@ def unstack(data: Dict[str, np.ndarray], /) -> TimeSeries:
 #     ts.sort_events()
 
 #     n_cycles = int(ts.time.shape[0] / n_points)
-#     out = {}  # type: Dict[str, np.ndarray]
+#     out = {}  # type: dict[str, np.ndarray]
 
 #     # Init
 #     for event in ts.events:
@@ -526,9 +550,9 @@ def unstack(data: Dict[str, np.ndarray], /) -> TimeSeries:
 #     return out
 
 
-def most_repeatable_cycles(data: np.ndarray, /) -> List[int]:
+def most_repeatable_cycles(data: ArrayLike, /) -> list[int]:
     """
-    Get the indexes of the most repeatable cycles in TimeSeries or array.
+    Get the indexes of the most repeatable cycles in array.
 
     This function returns an ordered list of the most repeatable to the least
     repeatable cycles.
@@ -550,8 +574,8 @@ def most_repeatable_cycles(data: np.ndarray, /) -> List[int]:
 
     Returns
     -------
-    List[int]
-        List of indexes corresponding to the cycles in most to least
+    list[int]
+        list of indexes corresponding to the cycles in most to least
         repeatable order.
 
     Example
@@ -570,10 +594,12 @@ def most_repeatable_cycles(data: np.ndarray, /) -> List[int]:
     [1, 3, 0, 2]
 
     """
-    data = data.copy()
+    check_types(most_repeatable_cycles, locals())
+
+    data = np.array(data)
     n_cycles = data.shape[0]
-    out_cycles = []  # type: List[int]
-    done_cycles = []  # type: List[int]  # Like out_cycles but includes NaNs
+    out_cycles = []  # type: list[int]
+    done_cycles = []  # type: list[int]  # Like out_cycles but includes NaNs
 
     # Exclude cycles with nans: put nans for all data of this cycle
     for i_cycle in range(n_cycles - 1, -1, -1):
