@@ -24,6 +24,8 @@ As a convention, the first dimension of every array is always N and corresponds
 to time.
 
 """
+from __future__ import annotations
+
 
 __author__ = "Félix Chénier"
 __copyright__ = "Copyright (C) 2020-2021 Félix Chénier"
@@ -47,10 +49,12 @@ def __dir__():
 import numpy as np
 import scipy.spatial.transform as transform
 import kineticstoolkit.external.icp as icp
-from typing import Optional, Tuple
+from typing import Optional
+from numpy.typing import ArrayLike
+from kineticstoolkit.exceptions import check_types
 
 
-def matmul(op1: np.ndarray, op2: np.ndarray) -> np.ndarray:
+def matmul(op1: ArrayLike, op2: ArrayLike) -> np.ndarray:
     """
     Matrix multiplication between series of matrices.
 
@@ -84,6 +88,10 @@ def matmul(op1: np.ndarray, op2: np.ndarray) -> np.ndarray:
            [16.,  9.]])
 
     """
+    check_types(matmul, locals())
+
+    op1 = np.array(op1)
+    op2 = np.array(op2)
 
     def perform_mul(op1, op2):
         if isinstance(op1, np.ndarray) and isinstance(op2, np.ndarray):
@@ -108,8 +116,8 @@ def matmul(op1: np.ndarray, op2: np.ndarray) -> np.ndarray:
 
 def create_transforms(
     seq: Optional[str] = None,
-    angles: Optional[np.ndarray] = None,
-    translations: Optional[np.ndarray] = None,
+    angles: Optional[ArrayLike] = None,
+    translations: Optional[ArrayLike] = None,
     degrees=False,
 ) -> np.ndarray:
     """
@@ -175,6 +183,8 @@ def create_transforms(
                 [ 0.,  0.,  0.,  1.]]])
 
     """
+    check_types(create_transforms, locals())
+
     # Condition translations
     if translations is None:
         translations = np.zeros((1, 3))
@@ -208,7 +218,7 @@ def create_transforms(
 
 
 def get_angles(
-    T: np.ndarray, seq: str, degrees: bool = False, flip: bool = False
+    T: ArrayLike, seq: str, degrees: bool = False, flip: bool = False
 ) -> np.ndarray:
     """
     Extract Euler angles from a series of homogeneous matrices.
@@ -270,6 +280,9 @@ def get_angles(
     `seq` parameter.
 
     """
+    check_types(get_angles, locals())
+
+    T = np.array(T)
     R = transform.Rotation.from_matrix(T[:, 0:3, 0:3])
     angles = R.as_euler(seq, degrees)
 
@@ -290,13 +303,13 @@ def get_angles(
 
 
 def create_frames(
-    origin: np.ndarray,
-    x: Optional[np.ndarray] = None,
-    y: Optional[np.ndarray] = None,
-    z: Optional[np.ndarray] = None,
-    xy: Optional[np.ndarray] = None,
-    xz: Optional[np.ndarray] = None,
-    yz: Optional[np.ndarray] = None,
+    origin: ArrayLike,
+    x: Optional[ArrayLike] = None,
+    y: Optional[ArrayLike] = None,
+    z: Optional[ArrayLike] = None,
+    xy: Optional[ArrayLike] = None,
+    xz: Optional[ArrayLike] = None,
+    yz: Optional[ArrayLike] = None,
 ) -> np.ndarray:
     """
     Create an Nx4x4 series of frames based on series of points and vectors.
@@ -307,21 +320,21 @@ def create_frames(
         A series of N points (Nx4) that corresponds to the origin of the
         series of frames to be created.
 
-    x|y|z
+    x, y, z
         Define either `x`, `y` or `z`. A series of N vectors (Nx4) that
         are aligned toward the {x|y|z} series of frames to be created.
 
-    xy|xz
+    xy, xz
         Only if `x` is specified. A series of N vectors (Nx4) in the {xy|xz}
         plane of the series of frames to be created. As a rule of thumb, use
         a series of N vectors that correspond roughly to the {z|-y} axis.
 
-    xy|yz
+    xy, yz
         Only if `y` is specified. A series of N vectors (Nx4) in the {xy|yz}
         plane of the series of frames to be created. As a rule of thumb, use
         a series of N vectors that correspond roughly to the {z|x} axis.
 
-    xz|yz
+    xz, yz
         Only if `z` is specified. A series of N vectors (Nx4) in the {xz|yz}
         plane of the series of frames to be created. As a rule of thumb, use
         a series of N vectors that correspond roughly to the {-y|x} axis.
@@ -336,6 +349,7 @@ def create_frames(
     ktk.geometry.create_transforms
 
     """
+    check_types(create_frames, locals())
 
     def normalize(v):
         """Normalize series of vectors."""
@@ -390,7 +404,7 @@ def create_frames(
 
 
 def get_local_coordinates(
-    global_coordinates: np.ndarray, reference_frames: np.ndarray
+    global_coordinates: ArrayLike, reference_frames: ArrayLike
 ) -> np.ndarray:
     """
     Express global coordinates in local reference frames.
@@ -420,6 +434,11 @@ def get_local_coordinates(
     ktk.geometry.get_global_coordinates
 
     """
+    check_types(get_local_coordinates, locals())
+
+    global_coordinates = np.array(global_coordinates)
+    reference_frames = np.array(reference_frames)
+
     n_samples = global_coordinates.shape[0]
 
     # Transform NaNs in global coordinates to zeros to perform the operation,
@@ -453,7 +472,7 @@ def get_local_coordinates(
 
 
 def get_global_coordinates(
-    local_coordinates: np.ndarray, reference_frames: np.ndarray
+    local_coordinates: ArrayLike, reference_frames: ArrayLike
 ) -> np.ndarray:
     """
     Express local coordinates in the global reference frame.
@@ -483,18 +502,23 @@ def get_global_coordinates(
     ktk.geometry.get_local_coordinates
 
     """
+    check_types(get_global_coordinates, locals())
+
+    local_coordinates = np.array(local_coordinates)
+    reference_frames = np.array(reference_frames)
+
     global_coordinates = np.zeros(local_coordinates.shape)
     global_coordinates = matmul(reference_frames, local_coordinates)
     return global_coordinates
 
 
-def isnan(input: np.ndarray, /) -> np.ndarray:
+def isnan(array: ArrayLike, /) -> np.ndarray:
     """
     Check which samples has at least one NaN.
 
     Parameters
     ----------
-    input
+    in
         Array where the first dimension corresponds to time.
 
     Returns
@@ -504,7 +528,9 @@ def isnan(input: np.ndarray, /) -> np.ndarray:
         True for the samples that contain at least one NaN.
 
     """
-    temp = np.isnan(input)
+    check_types(isnan, locals())
+
+    temp = np.isnan(array)
     while len(temp.shape) > 1:
         temp = temp.sum(axis=1) > 0
     return temp
@@ -512,7 +538,7 @@ def isnan(input: np.ndarray, /) -> np.ndarray:
 
 def _match_size(
     op1: np.ndarray, op2: np.ndarray
-) -> Tuple[np.ndarray, np.ndarray]:
+) -> tuple[np.ndarray, np.ndarray]:
     """
     Match the first dimension of op1 and op2.
 
@@ -523,7 +549,7 @@ def _match_size(
 
     Returns
     -------
-    Tuple of two np.ndarray
+    tuple of two np.ndarray
         op1 and op2, now matched in size.
 
     """
@@ -540,7 +566,7 @@ def _match_size(
 
 
 def register_points(
-    global_points: np.ndarray, local_points: np.ndarray
+    global_points: ArrayLike, local_points: ArrayLike
 ) -> np.ndarray:
     """
     Find the homogeneous transforms between two series of point clouds.
@@ -566,6 +592,11 @@ def register_points(
     parameters may change in the future.
 
     """
+    check_types(register_points, locals())
+
+    global_points = np.array(global_points)
+    local_points = np.array(local_points)
+
     n_samples = global_points.shape[0]
 
     # Prealloc the transformation matrix
