@@ -18,6 +18,8 @@
 """
 Provide functions related to kinematics analysis.
 """
+from __future__ import annotations
+
 
 __author__ = "Félix Chénier"
 __copyright__ = "Copyright (C) 2020 Félix Chénier"
@@ -28,8 +30,7 @@ __license__ = "Apache 2.0"
 import kineticstoolkit.geometry as geometry
 from kineticstoolkit import TimeSeries, read_c3d, write_c3d
 from kineticstoolkit.decorators import deprecated
-from typing import Sequence, Dict, Union
-from copy import deepcopy
+from kineticstoolkit.exceptions import check_types
 
 import numpy as np
 import warnings
@@ -44,122 +45,9 @@ def __dir__():
     ]
 
 
-# def read_xml_file(filename):
-#
-#    # Define a helper function that reads the file line by line until it finds
-#    # one of the strings in a given list.
-#    def read_until(list_strings):
-#        while True:
-#            one_line = fid.readline()
-#            if len(one_line) == 0:
-#                return(one_line)
-#
-#            for one_string in list_strings:
-#                if one_line.find(one_string) >= 0:
-#                    return(one_line)
-#
-#
-#    # Open the file
-#    if os.path.isfile(filename) == False:
-#        raise(FileNotFoundError)
-#
-#    fid = open(filename, 'r')
-#
-#    # Reading loop
-#    the_timeseries = []
-#
-#    while True:
-#
-#        # Wait for next label
-#        one_string = read_until(['>Label :</Data>'])
-#
-#        if len(one_string) > 0:
-#            # A new label was found
-#
-#            # Isolate the label name
-#            one_string = read_until(['<Data'])
-#            label_name = one_string[
-#                    (one_string.find('"String">')+9):one_string.find('</Data>')]
-#            print(label_name)
-#
-#            # Isolate the data format
-#            one_string = read_until(['>Coords'])
-#            one_string = one_string[
-#                    (one_string.find('<Data>')+6):one_string.find('</Data>')]
-#            data_unit = one_string[
-#                    (one_string.find('x,y:')+4):one_string.find('; ')]
-#
-#            # Ignore the next data lines (header)
-#            one_string = read_until(['<Data'])
-#            one_string = read_until(['<Data'])
-#            one_string = read_until(['<Data'])
-#
-#            # Find all data for this marker
-#            time = np.array([1.0]) # Dummy init
-#            data = np.zeros((1,2)) # Dummy init
-#            sample_index = 0
-#
-#            while(True):
-#
-#                # Find the next x data
-#                one_string = read_until(['<Data'])
-#                one_string = one_string[
-#                        (one_string.find('"Number">')+9):one_string.find('</Data>')]
-#
-#                try:
-#                    # If it's a float, then add it.
-#                    # Add a new row to time and data
-#                    if sample_index > 0:
-#                        time = np.block([time, np.array(1)])
-#                        data = np.block([[data], [np.zeros((1,2))]])
-#
-#                    data[sample_index, 0] = float(one_string)
-#
-#                except:
-#                    the_timeseries.append(ts.TimeSeries(name=label_name, data=data, time=time, dataunit=data_unit, timeunit='s'))
-#                    break #No data
-#
-#                # Find the next y data
-#                one_string = read_until(['<Data'])
-#                one_string = one_string[
-#                        (one_string.find('"Number">')+9):one_string.find('</Data>')]
-#                data[sample_index, 1] = float(one_string)
-#
-#                # Find the next t data
-#                one_string = read_until(['<Data'])
-#                one_string = one_string[
-#                        (one_string.find('">')+2):one_string.find('</Data>')]
-#
-#                if one_string.find(':') < 0:
-#                    time[sample_index] = float(one_string) # milliseconds or #frame
-#                else:
-#                    index = one_string.find(':')
-#                    hours = one_string[0:index]
-#                    one_string = one_string[index+1:]
-#
-#                    index = one_string.find(':')
-#                    minutes = one_string[0:index]
-#                    one_string = one_string[index+1:]
-#
-#                    index = one_string.find(':')
-#                    seconds = one_string[0:index]
-#                    milliseconds = one_string[index+1:]
-#
-#                    time[sample_index] = (3600. * float(hours) +
-#                        60. * float(minutes) + float(seconds) +
-#                        (int(milliseconds) % 1000) / 1000)
-#
-#
-#                sample_index = sample_index + 1
-#
-#        else:
-#            # EOF
-#            return(the_timeseries)
-
-
 def create_cluster(
-    markers: TimeSeries, /, marker_names: Sequence[str]
-) -> Dict[str, np.ndarray]:
+    markers: TimeSeries, /, names: list[str]
+) -> dict[str, np.ndarray]:
     """
     Create a cluster definition based on a static acquisition.
 
@@ -167,13 +55,13 @@ def create_cluster(
     ----------
     markers
         Markers trajectories during a static acquisition.
-    marker_names
+    names
         The markers that define the cluster.
 
     Returns
     -------
-    Dict
-        Dictionary where each entry represents the local position of a marker
+    dict
+        dictionary where each entry represents the local position of a marker
         in an arbitrary coordinate system.
 
     Warning
@@ -181,19 +69,23 @@ def create_cluster(
     This function, which has been introduced in 0.7, is still experimental and
     may change signature or behaviour in the future.
 
+    0.9.3: Parameters `marker_names` was changed to `names`
+
     See also
     --------
     ktk.kinematics.extend_cluster
     ktk.kinematics.track_cluster
 
     """
+    check_types(create_cluster, locals())
+
     n_samples = len(markers.time)
-    n_markers = len(marker_names)
+    n_markers = len(names)
 
     # Construct the global points array
     global_points = np.empty((n_samples, 4, n_markers))
 
-    for i_marker, marker in enumerate(marker_names):
+    for i_marker, marker in enumerate(names):
         global_points[:, :, i_marker] = markers.data[marker][:, :]
 
     # Remove samples where at least one marker is invisible
@@ -212,15 +104,15 @@ def create_cluster(
 
     # Create the output
     output = {}
-    for i_marker, marker_name in enumerate(marker_names):
-        output[marker_name] = local_points[:, :, i_marker]
+    for i_marker, name in enumerate(names):
+        output[name] = local_points[:, :, i_marker]
 
     return output
 
 
 def extend_cluster(
-    markers: TimeSeries, /, cluster: Dict[str, np.ndarray], new_point: str
-) -> Dict[str, np.ndarray]:
+    markers: TimeSeries, /, cluster: dict[str, np.ndarray], name: str
+) -> dict[str, np.ndarray]:
     """
     Add a point to an existing cluster.
 
@@ -231,12 +123,12 @@ def extend_cluster(
         trajectories from the cluster definition.
     cluster
         The source cluster to add a new point to.
-    new_point
+    name
         The name of the point to add (data key of the markers TimeSeries).
 
     Returns
     -------
-    Dict[str, np.ndarray]
+    dict[str, np.ndarray]
         A copy of the initial cluster, with the added point.
 
     Warning
@@ -244,25 +136,34 @@ def extend_cluster(
     This function, which has been introduced in 0.7, is still experimental and
     may change signature or behaviour in the future.
 
+    0.9.3: Parameter `new_point` was changed to `name`
+
     See also
     --------
     ktk.kinematics.create_cluster
     ktk.kinematics.track_cluster
 
     """
-    cluster = deepcopy(cluster)
+    check_types(extend_cluster, locals())
+
+    # Ensure to convert every cluster element to a numpy array
+    new_cluster = {}
+    for key in cluster:
+        new_cluster[key] = np.array(cluster[key])
+    cluster = new_cluster
+
     frames = _track_cluster_frames(markers, cluster)
     local_coordinates = geometry.get_local_coordinates(
-        markers.data[new_point], frames
+        markers.data[name], frames
     )
-    cluster[new_point] = np.nanmean(local_coordinates, axis=0)[np.newaxis]
+    cluster[name] = np.nanmean(local_coordinates, axis=0)[np.newaxis]
     return cluster
 
 
 def track_cluster(
     markers: TimeSeries,
     /,
-    cluster: Dict[str, np.ndarray],
+    cluster: dict[str, np.ndarray],
     *,
     include_lcs: bool = False,
     lcs_name: str = "LCS",
@@ -303,6 +204,8 @@ def track_cluster(
     ktk.kinematics.track_cluster
 
     """
+    check_types(track_cluster, locals())
+
     out = markers.copy(copy_data=False, copy_data_info=False)
     unit = _get_marker_unit(markers)
 
@@ -323,12 +226,14 @@ def track_cluster(
 
 
 def _track_cluster_frames(
-    markers: TimeSeries, cluster: Dict[str, np.ndarray]
+    markers: TimeSeries, cluster: dict[str, np.ndarray]
 ) -> np.ndarray:
     """Track a cluster and return its frame series."""
     # Set local and global points
     marker_names = cluster.keys()
-    stacked_local_points = np.dstack([cluster[_] for _ in marker_names])
+    stacked_local_points = np.dstack(
+        [np.array(cluster[_]) for _ in marker_names]
+    )
 
     global_points = np.empty(
         (len(markers.time), 4, stacked_local_points.shape[2])
@@ -349,7 +254,7 @@ def _track_cluster_frames(
     return frames
 
 
-def _get_marker_unit(markers: TimeSeries) -> Union[None, str]:
+def _get_marker_unit(markers: TimeSeries) -> None | str:
     """Get markers unit, raise ValueError if not all have the same unit."""
     unit = None
     for marker in markers.data:
@@ -370,7 +275,7 @@ def _get_marker_unit(markers: TimeSeries) -> Union[None, str]:
     return unit
 
 
-def write_trc_file(markers: TimeSeries, filename: str) -> None:
+def write_trc_file(markers: TimeSeries, /, filename: str) -> None:
     """
     Export a markers TimeSeries to OpenSim's TRC file format.
 
@@ -389,6 +294,8 @@ def write_trc_file(markers: TimeSeries, filename: str) -> None:
     developed.
 
     """
+    check_types(write_trc_file, locals())
+
     markers = markers.copy()
     markers.fill_missing_samples(0)
 
@@ -439,8 +346,8 @@ def write_trc_file(markers: TimeSeries, filename: str) -> None:
     since="master", until="January 2023", details="Use create_cluster()."
 )
 def define_rigid_body(
-    kinematics: TimeSeries, marker_names: Sequence[str]
-) -> Dict[str, np.ndarray]:
+    kinematics: TimeSeries, marker_names: list[str]
+) -> dict[str, np.ndarray]:
     """
     Create a generic rigid body definition based on a static acquisition.
 
@@ -453,8 +360,8 @@ def define_rigid_body(
 
     Returns
     -------
-    Dict
-        Dictionary where each entry represents the local position of a point
+    dict
+        dictionary where each entry represents the local position of a point
         (e.g., marker name). The key is the name of the point, the value is a
         1x4 array that indicates the local position of this point in the rigid
         body's local coordinate system.
@@ -469,7 +376,7 @@ def define_rigid_body(
 def track_rigid_body(
     kinematics: TimeSeries,
     /,
-    local_points: Dict[str, np.ndarray],
+    local_points: dict[str, np.ndarray],
     label: str = "Trajectory",
     *,
     include_rigid_body: bool = True,
@@ -521,7 +428,7 @@ def track_rigid_body(
 def track_rigid_bodies(
     markers: TimeSeries,
     /,
-    definitions: Dict[str, Dict[str, np.ndarray]],
+    definitions: dict[str, dict[str, np.ndarray]],
     *,
     include_rigid_bodies: bool = True,
     include_markers: bool = False,
@@ -693,7 +600,7 @@ def write_c3d_file(filename: str, markers: TimeSeries) -> None:
     until="2024",
     details=("This function has been moved to the n3d extension."),
 )
-def read_n3d_file(filename: str, labels: Sequence[str] = []) -> TimeSeries:
+def read_n3d_file(filename: str, labels: list[str] = []) -> TimeSeries:
     """
     Read markers from an NDI N3D file.
 
