@@ -21,6 +21,8 @@ Provides the Player class to visualize markers and rigid bodies in 3d.
 The Player class is accessible directly from the toplevel Kinetics Toolkit
 namespace (i.e., ktk.Player).
 """
+from __future__ import annotations
+
 
 __author__ = "Félix Chénier"
 __copyright__ = "Copyright (C) 2020 Félix Chénier"
@@ -38,7 +40,8 @@ import numpy as np
 from numpy import sin, cos
 import time
 import copy
-from typing import Union, Sequence, Dict, Any, Optional
+from typing import Any, Optional
+from numpy.typing import ArrayLike
 
 # To fit the new viewpoint on selecting a new marker
 import scipy.optimize as optim
@@ -83,13 +86,14 @@ class Player:
         Optional. Each key corresponds to an inerconnection between markers,
         where one interconnection is another dict with the following keys:
 
-        - ``Links``: list of lists of 2 strings, where each string is a marker
-          name. For example, to link Marker1 to Marker2 and
-          Marker1 to Marker3, Links would be::
+        - 'Links': list of lists strings, where each string is a marker
+          name. For example, to create a link that spans Marker1 and Marker2,
+          and another link that spans Marker3, Marker4 and Marker5,
+          interconnections['Links'] would be::
 
-              [['Marker1', 'Marker2'], ['Marker1', 'Marker3']]
+              [['Marker1', 'Marker2'], ['Marker3', 'Marker4', 'Marker5']]
 
-        - ``Color``: character or tuple that represents the color of the
+        - 'Color': character or tuple (RGB) that represents the color of the
           link. Color must be a valid value for matplotlib's
           plots.
 
@@ -138,7 +142,7 @@ class Player:
     def __init__(
         self,
         *ts: TimeSeries,
-        interconnections: Dict[str, Dict[str, Any]] = {},
+        interconnections: dict[str, dict[str, Any]] = {},
         current_index: int = 0,
         marker_radius: float = 0.008,
         axis_length: float = 0.1,
@@ -148,8 +152,8 @@ class Player:
         zoom: float = 1.0,
         azimuth: float = 0.0,
         elevation: float = 0.2,
-        translation: Union[Sequence[float], np.ndarray] = (0.0, 0.0),
-        target: Union[Sequence[float], np.ndarray] = (0.0, 0.0, 0.0),
+        translation: ArrayLike = (0.0, 0.0),
+        target: ArrayLike = (0.0, 0.0, 0.0),
         track: bool = False,
         perspective: bool = True,
         **kwargs,
@@ -220,27 +224,17 @@ class Player:
         #   /
         # z/
         if up == "x":
-            rotation = geometry.create_transforms(
-                "z", np.array([90]), degrees=True
-            )
+            rotation = geometry.create_transforms("z", [90], degrees=True)
         elif up == "y":
             rotation = np.eye(4)[np.newaxis]
         elif up == "z":
-            rotation = geometry.create_transforms(
-                "x", np.array([-90]), degrees=True
-            )
+            rotation = geometry.create_transforms("x", [-90], degrees=True)
         elif up == "-x":
-            rotation = geometry.create_transforms(
-                "z", np.array([-90]), degrees=True
-            )
+            rotation = geometry.create_transforms("z", [-90], degrees=True)
         elif up == "-y":
-            rotation = geometry.create_transforms(
-                "z", np.array([-180]), degrees=True
-            )
+            rotation = geometry.create_transforms("z", [-180], degrees=True)
         elif up == "-z":
-            rotation = geometry.create_transforms(
-                "x", np.array([90]), degrees=True
-            )
+            rotation = geometry.create_transforms("x", [90], degrees=True)
         else:
             raise ValueError(
                 "up must be in {'x', 'y', 'z', '-x', '-y', '-z'}."
@@ -254,6 +248,11 @@ class Player:
                 self.rigid_bodies.data[key], rotation
             )
 
+        # Camera target
+        target1x4 = np.ones((1, 4))
+        target1x4[0, 0:3] = target
+        self.target = geometry.get_global_coordinates(target1x4, rotation)[0]
+
         # ---------------------------------------------------------------
         # Assign the interconnections
         self.interconnections = interconnections
@@ -268,14 +267,13 @@ class Player:
         self.zoom = zoom
         self.azimuth = azimuth
         self.elevation = elevation
-        self.target = target
         self.track = track
-        self.translation = translation
+        self.translation = np.array(translation)
         self.perspective = perspective
         self.playback_speed = 1.0  # 0.0 to show all samples
         #  self.anim = None  # Will initialize in _create_figure
 
-        self.objects = dict()  # type: Dict[str, Any]
+        self.objects = dict()  # type: dict[str, Any]
         self._colors = ["r", "g", "b", "y", "c", "m", "w"]
         self.objects["PlotMarkers"] = dict()
         for color in self._colors:
@@ -291,7 +289,7 @@ class Player:
         self.objects["Axes"] = None
         self.objects["Help"] = None
 
-        self.state = dict()  # type: Dict[str, Any]
+        self.state = dict()  # type: dict[str, Any]
         self.state["ShiftPressed"] = False
         self.state["MouseLeftPressed"] = False
         self.state["MouseMiddlePressed"] = False
@@ -615,34 +613,25 @@ class Player:
         # Draw the interconnections
         if self.interconnections is not None:
             for interconnection in self.interconnections:
-                n_links = len(self.interconnections[interconnection]["Links"])
-                coordinates = np.empty((3 * n_links, 4))
-                coordinates[:] = np.nan
-                for i_link in range(n_links):
-                    marker1 = self.interconnections[interconnection]["Links"][
-                        i_link
-                    ][0]
-                    marker2 = self.interconnections[interconnection]["Links"][
-                        i_link
-                    ][1]
-                    if marker1 in interconnection_markers:
-                        coordinates[3 * i_link] = interconnection_markers[
-                            marker1
-                        ]
-                    else:
-                        coordinates[3 * i_link] = np.nan
 
-                    if marker2 in interconnection_markers:
-                        coordinates[3 * i_link + 1] = interconnection_markers[
-                            marker2
-                        ]
-                    else:
-                        coordinates[3 * i_link + 1] = np.nan
+                coordinates = []
+                chains = self.interconnections[interconnection]["Links"]
 
-                coordinates = self._get_projection(coordinates)
+                for chain in chains:
+                    for marker in chain:
+
+                        try:
+                            coordinates.append(interconnection_markers[marker])
+                        except KeyError:
+                            coordinates.append(np.repeat(np.nan, 4))
+
+                    coordinates.append(np.repeat(np.nan, 4))
+
+                np_coordinates = np.array(coordinates)
+                np_coordinates = self._get_projection(np_coordinates)
 
                 self.objects["PlotInterconnections"][interconnection].set_data(
-                    coordinates[:, 0], coordinates[:, 1]
+                    np_coordinates[:, 0], np_coordinates[:, 1]
                 )
 
     def _update_plots(self) -> None:
@@ -737,10 +726,9 @@ class Player:
 
         self.objects["Figure"].canvas.draw()
 
-    def _set_new_target(
-        self, target: Union[Sequence[float], np.ndarray]
-    ) -> None:
+    def _set_new_target(self, target: ArrayLike) -> None:
         """Set new target and adapts translation and zoom consequently."""
+        target = np.array(target)
         if np.sum(np.isnan(target)) > 0:
             return
         initial_translation = copy.deepcopy(self.translation)
