@@ -16,6 +16,8 @@
 # limitations under the License.
 
 """Provide miscelleanous helper functions."""
+from __future__ import annotations
+
 
 __author__ = "Félix Chénier"
 __copyright__ = "Copyright (C) 2020 Félix Chénier"
@@ -27,6 +29,7 @@ import kineticstoolkit.config
 import kineticstoolkit._repr as _repr
 import kineticstoolkit.gui
 import kineticstoolkit.ext
+from kineticstoolkit.exceptions import check_types
 
 import warnings
 
@@ -93,6 +96,8 @@ def change_defaults(
         import kineticstoolkit.lab as ktk
 
     """
+    check_types(change_defaults, locals())
+
     if change_ipython_dict_repr:
         # Modify the repr function for dicts in IPython
         try:
